Package: COMPASS
Type: Package
Title: Combinatorial Polyfunctionality Analysis of Single Cells
Version: 1.49.0
Date: 2014-07-11
Author: Lynn Lin, Kevin Ushey, Greg Finak, Ravio Kolde (pheatmap)
Authors@R: c( person("Lynn", "Lin", role="aut", email="llin@fhcrc.org"),
    person("Kevin", "Ushey", role="aut", email="kushey@fhcrc.org"),
    person("Greg", "Finak", role=c("aut","cre"), email="gfinak@fhcrc.org"),
    person("Raivo", "Kolde", role="ctb", email="rkolde@gmail.com",
    comment="Author of 'pheatmap', which was modified and now used internally
    in COMPASS") )
Description: COMPASS is a statistical framework that enables unbiased analysis
    of antigen-specific T-cell subsets. COMPASS uses a Bayesian hierarchical
    framework to model all observed cell-subsets and select the most likely
    to be antigen-specific while regularizing the small cell counts that often
    arise in multi-parameter space. The model provides a posterior probability
    of specificity for each cell subset and each sample, which can be used to
    profile a subject's immune response to external stimuli such as infection
    or vaccination.
License: Artistic-2.0
BugReports: https://github.com/RGLab/COMPASS/issues
VignetteBuilder: knitr
Depends: R (>= 3.0.3)
LinkingTo: Rcpp (>= 0.11.0)
Maintainer: Greg Finak <gfinak@fhcrc.org>
Imports: methods, Rcpp, data.table, RColorBrewer, scales, grid, plyr,
        knitr, abind, clue, grDevices, utils, pdist, magrittr,
        reshape2, dplyr, tidyr, rlang, BiocStyle, rmarkdown, foreach,
        coda
Suggests: flowWorkspace (>= 3.33.1), flowCore, ncdfFlow, shiny,
        testthat, devtools, flowWorkspaceData, ggplot2, progress
LazyLoad: yes
LazyData: yes
biocViews: ImmunoOncology, FlowCytometry
Encoding: UTF-8
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/COMPASS
git_branch: devel
git_last_commit: 43c7ae7
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-16
NeedsCompilation: yes
Packaged: 2025-11-17 02:41:04 UTC; biocbuild
Built: R 4.6.0; x86_64-apple-darwin20; 2025-11-17 11:50:12 UTC; unix
