# Generated by roxygen2: do not edit by hand

S3method("metadata<-",COMPASSContainer)
S3method(CellCounts,COMPASSContainer)
S3method(CellCounts,default)
S3method(FunctionalityScore,COMPASSResult)
S3method(FunctionalityScore,default)
S3method(PolyfunctionalityScore,COMPASSResult)
S3method(PolyfunctionalityScore,default)
S3method(Response,COMPASSResult)
S3method(UniqueCombinations,COMPASSContainer)
S3method(UniqueCombinations,default)
S3method(markers,COMPASSContainer)
S3method(markers,COMPASSResult)
S3method(melt_,data.frame)
S3method(melt_,matrix)
S3method(merge,COMPASSContainer)
S3method(metadata,COMPASSContainer)
S3method(metadata,COMPASSResult)
S3method(plot,COMPASSResult)
S3method(print,COMPASSContainer)
S3method(print,COMPASSResult)
S3method(subset,COMPASSContainer)
S3method(summary,COMPASSContainer)
S3method(summary,COMPASSResult)
export("COMPASSDescription<-")
export("metadata<-")
export(COMPASS)
export(COMPASSContainer)
export(COMPASSContainerFromGatingSet)
export(COMPASSDescription)
export(COMPASSMCMCDiagnosis)
export(COMPASSfitToCountsTable)
export(CellCounts)
export(Combinations)
export(FunctionalityScore)
export(GetThresholdedIntensities)
export(PolyfunctionalityScore)
export(Posterior)
export(PosteriorDiff)
export(PosteriorLogDiff)
export(PosteriorPs)
export(PosteriorPu)
export(Response)
export(SimpleCOMPASS)
export(TotalCellCounts)
export(UniqueCombinations)
export(categories)
export(getCounts)
export(markers)
export(metadata)
export(pheatmap)
export(plot2)
export(plotCOMPASSResultStack)
export(scores)
export(select_compass_pops)
export(shinyCOMPASS)
export(shinyCOMPASSDeps)
export(translate_marker_names)
export(transpose_list)
import(data.table)
import(grid)
import(magrittr)
import(methods)
import(pdist)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rcpp,evalCpp)
importFrom(abind,abind)
importFrom(clue,solve_LSAP)
importFrom(dplyr,bind_rows)
importFrom(dplyr,enquo)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(grDevices,bmp)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,hsv)
importFrom(grDevices,jpeg)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grDevices,rgb2hsv)
importFrom(grDevices,tiff)
importFrom(graphics,hist)
importFrom(graphics,strwidth)
importFrom(knitr,kable)
importFrom(methods,is)
importFrom(plyr,.)
importFrom(plyr,ddply)
importFrom(plyr,laply)
importFrom(plyr,ldply)
importFrom(plyr,rbind.fill)
importFrom(rlang,"!!")
importFrom(scales,div_gradient_pal)
importFrom(stats,as.dist)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(tidyr,"%>%")
importFrom(tidyr,gather)
importFrom(utils,adist)
importFrom(utils,combn)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,installed.packages)
importFrom(utils,sessionInfo)
useDynLib(COMPASS, .registration=TRUE)
